/* 
   PkgsSel-show 
   Copyright (C) 1999 Jarek Woloszyn (yossa@pld.org.pl)
   $Id: pkgssel-show.c,v 1.2 2001/02/11 17:35:57 yossa Exp $

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <newt.h>
#include <trurl/trurl.h>
#include <rpmpldi.h>
#include <newt-addon.h>

#include "pkgssel-data.h"
#include "pkgssel-show.h"
#include "regexp-search.h"
#include "defines.h"

extern unsigned LISTWIDTH;
extern unsigned MAINWINWIDTH;

/* Sets one label component with text and value */
void pkgsSetLabel(newtComponent co, char *text, long val, int size_type)
{
    char tmpbuf[2][100] =
    {"", ""};
    char unit[5], templ[20];
    long d;			/* divider */
    long tmp;
    int i, f;
    /*  Depending from size_type we use bytes, kilobytes, megabytes */

    switch (size_type) {
    case SIZE_B:
    case SIZE_KB:
	if (size_type == SIZE_B) {
	    d = 1;
	    strcpy(unit, _("bytes"));
	} else {
	    d = 1024;
	    strcpy(unit, "kB");
	}
	tmp = val / d;
	f = i = 0;
	if (!tmp)
	    strcpy(tmpbuf[1 - i], "0");
	while (tmp > 0) {
	    if (tmp / 1000)
		strcpy(templ, "%03li%s%s");
	    else
		strcpy(templ, "%li%s%s");

	    sprintf(tmpbuf[i], templ, tmp % 1000, (f ? "," : ""),
		    tmpbuf[1 - i]);
	    tmp /= 1000;
	    f = 1;
	    i = 1 - i;
	}
	sprintf(tmpbuf[i], "%s%s %s", text, tmpbuf[1 - i], unit);
	strcpy(tmpbuf[1 - i], tmpbuf[i]);
	break;
    case SIZE_MB:
	d = 1024 * 1024;
	strcpy(unit, "MB");
	sprintf(tmpbuf[0], "%s%.2f %s", text, (float) val / d, unit);
	break;
    case SIZE_NONE:
	sprintf(tmpbuf[0], "%s%li", text, val);
	break;
    }
    newtLabelSetText(co, tmpbuf[0]);
}

/* Sets 4 labels */
void pkgsSet4Labels(newtComponent * co, long val_num,
		    long val_size, long val_sel, long val_selsize,
		    int size_type)
{
    pkgsSetLabel(co[0], _("  Packages:      "), val_num, SIZE_NONE);
    pkgsSetLabel(co[1], _("  Size:          "), val_size, size_type);
    pkgsSetLabel(co[2], _("  Selected:      "), val_sel, SIZE_NONE);
    pkgsSetLabel(co[3], _("  Selected Size: "), val_selsize, size_type);
}

/* Counts the size of selected packages in current directory */
/* Args: components where it will put the values */
void pkgsCountDir(tn_array * actDir, newtComponent * co, int size_type)
{
    int i;
    int *tmp;
    long num, sel;
    long size, selsize;
    TSelectorItem *item;
    num = sel = size = selsize = 0;
    for (i = 0; i < n_array_size(actDir); i++) {
	item = n_array_nth(actDir, i);
	if ((item->type & SEL_PACKET) == SEL_PACKET) {
	    num++;
	    tmp = (int *) rpmpldi_header_entry_get(item->pkgnum,
						   RPMTAG_SIZE);
	    size += (tmp ? *tmp : 0);
	    if (*item->selptr) {
		sel++;
		selsize += (tmp ? *tmp : 0);
	    }
	    rpmpldi_header_entry_free(tmp);
	}
    }
    pkgsSet4Labels(co, num, size, sel, selsize, size_type);
}

/* Counts size of all selected packages */
/* Args: components where it will put the values */
void pkgsCountAllDirs(newtComponent * co, int size_type)
{
    int i;
    int *tmp;
    long num, sel;
    long size, selsize;
    TPkgSet *pkgsSet;
    pkgsSet = rpmpldi_getPkgsSet();
    num = pkgs_array_num(pkgsSet);
    sel = size = selsize = 0;
    for (i = 0; i < num; i++) {
	tmp = (int *) rpmpldi_header_entry_get(i, RPMTAG_SIZE);
	size += (tmp ? *tmp : 0);
	if (pkgs_array_nth(pkgsSet, i)->selected) {
	    sel++;
	    selsize += (tmp ? *tmp : 0);
	}
	rpmpldi_header_entry_free(tmp);
    }
    pkgsSet4Labels(co, num, size, sel, selsize, size_type);

}

/* Makes changes to the subdir_name
 * when where is >0 it just adds nameptr
 * when where is <0 it cuts off last directory level
 */

/* leaves subdir_name unchanged, instead changes the tmpbuf */
char *pkgsChangeDir(char *subdir_name, char *nameptr, int where)
{
    char tmpbuf[256];
    char *tmpptr;
    if (where < 0) {
	tmpptr = rindex(subdir_name, '/');
	n_assert(tmpptr != NULL);
	tmpptr[0] = 0;
	tmpptr = rindex(subdir_name, '/');
	n_assert(tmpptr != NULL);
	tmpptr[1] = 0;
    } else if (where > 0) {
	strcat(subdir_name, nameptr);
	strcat(subdir_name, "/");
    }
    if (strlen(subdir_name) > MAINWINWIDTH - 2) {
	strcpy(tmpbuf, "...");
	strcat(tmpbuf,
	       subdir_name + (strlen(subdir_name) - MAINWINWIDTH + 5));
    } else
	strcpy(tmpbuf, subdir_name);
    return strdup(tmpbuf);
}


/* returns empty string. it's a static variable, so strdup it */
char *pkgsMkEmptyString(char c, int length)
{
    static char buf[256];
    if (length > 255)
	length = 255;
    memset(buf, c, 256 * sizeof(char));
    buf[length] = 0;
    return buf;
}
